@@ -1,6 +1,6 @@
 from ..station import Station, labware_loader, instrument_loader
 from ..geometry import LysisTube
-from ..utils import mix_bottom_top
+from ..utils import mix_bottom_top, WellWithVolume
 from itertools import chain, islice, repeat
 import math
 import logging
@@ -13,6 +13,7 @@ class StationA(Station):
             air_gap_dest_multi: float = 2,
             air_gap_sample: float = 20,
             air_gap_lys: float = 10,
+            deepwell_headroom_bottom: float = 1,
             dest_headroom_height: float = 2,
             dest_top_height: float = 5,
             dest_multi_headroom_height: float = 2,
@@ -22,7 +23,6 @@ class StationA(Station):
             hover_height: float = -2,
             ic_capacity: float = 180,
             ic_lys_headroom: float = 1.1,
-            ic_headroom_bottom: float = 2,
             ic_mix_repeats: int = 5,
             ic_mix_volume: float = 20,
             iec_volume: float = 19,
@@ -35,6 +35,7 @@ class StationA(Station):
             lysis_in_controls: bool = False,
             lysis_rate_aspirate: float = 100,
             lysis_rate_dispense: float = 100,
+            lysis_rate_mix: float = 100,
             lysis_volume: float = 160,
             lys_mix_repeats: int = 10,
             lys_mix_volume: float = 100,
@@ -77,7 +78,7 @@ class StationA(Station):
         :param hover_height: height from the top at which to hover (should be negative) in mm
         :param ic_capacity: capacity of the internal control tube
         :param ic_lys_headroom: headroom for the internal control and lysis tube (multiplier)
-        :param ic_headroom_bottom: headroom always to keep from the bottom of the internal control tube in mm
+        :param deepwell_headroom_bottom: headroom always to keep from the bottom of the deepwell in mm
         :param ic_mix_repeats: number of repetitions during mixing the internal control
         :param ic_mix_volume: volume aspirated for mixing the internal control in uL
         :param iec_volume: The volume of lysis buffer to use per sample in uL
@@ -136,7 +137,7 @@ class StationA(Station):
         self._dest_top_height = dest_top_height
         self._hover_height = hover_height
         self._ic_capacity = ic_capacity
-        self._ic_headroom_bottom = ic_headroom_bottom
+        self._deepwell_headroom_bottom = deepwell_headroom_bottom
         self._ic_lys_headroom = ic_lys_headroom
         self._ic_mix_repeats = ic_mix_repeats
         self._ic_mix_volume = ic_mix_volume
@@ -148,6 +149,7 @@ class StationA(Station):
         self._lysis_in_controls = lysis_in_controls
         self._lysis_rate_aspirate = lysis_rate_aspirate
         self._lysis_rate_dispense = lysis_rate_dispense
+        self._lysis_rate_mix = lysis_rate_mix
         self._lysis_volume = lysis_volume
         self._lys_mix_repeats = lys_mix_repeats
         self._lys_mix_volume = lys_mix_volume
@@ -287,6 +289,13 @@ class StationA(Station):
         self._lysis_tube.height -= self._ic_lys_headroom
 
     def transfer_sample(self, source, dest):
+        self._p_main.flow_rate.aspirate = self._sample_aspirate
+        self._p_main.flow_rate.dispense = self._sample_dispense
+
+        dest_well_with_volume = WellWithVolume(dest,
+                                               self._lysis_volume if self._lysis_first else 0,
+                                               self._dest_headroom_height)
+
         self.logger.debug("transferring from {} to {}".format(source, dest))
         self.pick_up(self._p_main)
 
@@ -307,16 +316,17 @@ class StationA(Station):
         self._p_main.air_gap(self._air_gap_sample)
 
         self._p_main.dispense(self._air_gap_sample, dest.top(self._hover_height))
-        self._p_main.dispense(self._sample_volume, dest.bottom(self._dest_top_height))
+        self._p_main.dispense(self._sample_volume, dest.bottom(dest_well_with_volume.height))
+        dest_well_with_volume.fill(self._sample_volume)
 
         if self._lysis_first:
-            self._p_main.flow_rate.aspirate = self._lysis_rate_aspirate
-            self._p_main.flow_rate.dispense = self._lysis_rate_dispense
+            self._p_main.flow_rate.aspirate = self._lysis_rate_mix
+            self._p_main.flow_rate.dispense = self._lysis_rate_mix
 
             # Mix with lysis buffer
             mix_bottom_top(
                 self._p_main, self._lys_mix_repeats, self._lys_mix_volume,
-                dest.bottom, self._dest_headroom_height, self._dest_top_height
+                dest.bottom, self._dest_headroom_height, dest_well_with_volume.height
             )
 
             self._p_main.flow_rate.aspirate = self._sample_aspirate
@@ -360,15 +370,15 @@ class StationA(Station):
         return dests
 
     def transfer_samples(self):
-        self._p_main.flow_rate.aspirate = self._sample_aspirate
-        self._p_main.flow_rate.dispense = self._sample_dispense
-
         n = len(list(self.non_control_positions()))
         for i, (s, d) in enumerate(self.non_control_positions()):
             if self.run_stage("transfer sample {}/{}".format(i + 1, n)):
                 self.transfer_sample(s, d)
 
     def transfer_lys(self):
+        self._p_main.flow_rate.aspirate = self._lysis_rate_aspirate
+        self._p_main.flow_rate.dispense = self._lysis_rate_dispense
+
         num_samples_per_aspirate = self._p_main.max_volume // self._lysis_volume
         self.logger.info(
             "1 aspirate can dispense {} times of {}ul.".format(num_samples_per_aspirate, self._lysis_volume))
@@ -404,16 +414,13 @@ class StationA(Station):
             self.drop(self._p_main)
 
     def transfer_internal_control(self, idx: int, dest):
-        self._p_main.flow_rate.aspirate = self._lysis_rate_aspirate
-        self._p_main.flow_rate.dispense = self._lysis_rate_dispense
-
         strip_ind = idx // self.cols_per_strip
         self.logger.debug(
             "transferring internal control strip {}/{} to {}".format(strip_ind + 1, self.num_ic_strips, dest))
         internal_control = self._strips_block.rows()[0][strip_ind]
         self.pick_up(self._m20)
         # no air gap to use 1 transfer only avoiding drop during multiple transfers
-        self._m20.transfer(self._iec_volume, internal_control, dest.bottom(self._ic_headroom_bottom), new_tip='never')
+        self._m20.transfer(self._iec_volume, internal_control, dest.bottom(self._deepwell_headroom_bottom), new_tip='never')
         self._m20.mix(self._ic_mix_repeats, self._ic_mix_volume, dest.bottom(self._dest_multi_headroom_height))
         self._m20.air_gap(self._air_gap_dest_multi)
         self.drop(self._m20)
