@@ -1,15 +1,14 @@
 import json
-import logging
 
-from covmatic_stations.station import instrument_loader, labware_loader
-from covmatic_stations.utils import uniform_divide, WellWithVolume
+from ..station import instrument_loader, labware_loader
+from ..utils import uniform_divide, WellWithVolume, MoveWithSpeed, mix_bottom_top
 from .technogenetics import StationBTechnogenetics
 from ..paired_pipette import PairedPipette
 from opentrons.types import Point
 from itertools import repeat
-import math
 import os
 
+
 class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
 
     def __init__(self,
@@ -30,14 +29,22 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
             **kwargs)
         self._pick_up_single = pick_up_single       # Force every paired pickup to be done with one pipette at a time.
 
-    @labware_loader(8, "_res12")
-    def load_res12(self):
+    @property
+    def _res12_labware_def(self):
         # re-loading labware to avoid bug on reagent reservoir (see https://github.com/Opentrons/opentrons/issues/7793)
         reservoir_file = os.path.join(os.path.split(__file__)[0], "nest_12_reservoir_15ml_modified.json")
         self.logger.info("Loading reservoir labware from: {}".format(reservoir_file))
         with open(reservoir_file) as labware_file:
             labware_def = json.load(labware_file)
-        self._res12 = self._ctx.load_labware_from_definition(labware_def, 5, 'Trough with WashReagents')
+        return labware_def
+
+    @labware_loader(8, "_res12")
+    def load_res12(self):
+        self._res12 = self._ctx.load_labware_from_definition(self._res12_labware_def, 5, 'Trough with WashReagent A')
+
+    @labware_loader(9, "_elut12")
+    def load_elut12(self):
+        self._elut12 = self._ctx.load_labware_from_definition(self._res12_labware_def, 2, 'Trough with Wash B and Elution buffer')
 
     @property
     def wash1(self):
@@ -45,7 +52,7 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
 
     @property
     def wash2(self):
-        return self._res12.rows()[0][-6:]
+        return self._elut12.rows()[0][:6]
 
     @property
     def water(self):
@@ -67,12 +74,24 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
         super(StationBTechnogeneticsPairedPipette, self).body()
 
     def mix_samples(self):
-        with PairedPipette(self._magplate, self.mag_samples_m, start_at="mix sample") as tp:
-            tp.set_flow_rate(aspirate=self._mix_samples_rate_aspirate, dispense=self._mix_samples_rate_dispense)
+        well_with_volume = WellWithVolume(self.temp_samples_m[0],
+                                          initial_vol=self._starting_vol - self._sample_mix_vol,
+                                          min_height=self._sample_mix_height,
+                                          headroom_height=0)
+        with PairedPipette(self._tempplate, self.temp_samples_m, start_at="mix sample") as tp:
+            tp.set_flow_rate(aspirate=self._mix_samples_rate, dispense=self._mix_samples_rate)
             tp.pick_up()
-            tp.mix(self._sample_mix_times, self._sample_mix_vol,
-                   locationFrom="target",
-                   well_modifier="bottom({})".format(self._sample_mix_height))
+            # Custom mix_bottom_top
+            for i in range(self._sample_mix_times):
+                if i + 1 == self._sample_mix_times and self._mix_samples_last_rate is not None:
+                    tp.set_flow_rate(dispense=self._mix_samples_last_rate)
+                tp.aspirate(self._sample_mix_vol,
+                            locationFrom="target",
+                            well_modifier="bottom({})".format(self._sample_mix_height))
+                tp.dispense(self._sample_mix_vol,
+                            locationFrom="target",
+                            well_modifier="bottom({})".format(well_with_volume.height))
+            tp.move_to(locationFrom="target", well_modifier="top(0)", speed=self._sample_vertical_speed)
             tp.air_gap(self._bind_air_gap)
             tp.drop_tip()
 
@@ -164,7 +183,7 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
             tp.dispense(self._elution_vol, locationFrom="side_locs")
             if self._elute_mix_times > 0:
                 tp.mix(self._elute_mix_times, self._elute_mix_vol, locationFrom="side_locs")
-            tp.touch_tip(v_offset=self._touch_tip_height)
+            # tp.touch_tip(v_offset=self._touch_tip_height)
             tp.air_gap(self._elute_air_gap)
             tp.drop_tip()
 
@@ -197,11 +216,13 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
             self._m300r.flow_rate.dispense = self._wash_1_mix_dispense_rate
             self._m300.flow_rate.aspirate = self._wash_1_mix_aspiration_rate
             self._m300r.flow_rate.aspirate = self._wash_1_mix_aspiration_rate
+            vertical_speed = self._wash_1_vertical_speed
         elif wash_name == "wash B":
             self._m300.flow_rate.dispense = self._wash_2_mix_dispense_rate
             self._m300r.flow_rate.dispense = self._wash_2_mix_dispense_rate
             self._m300.flow_rate.aspirate = self._wash_2_mix_aspiration_rate
             self._m300r.flow_rate.aspirate = self._wash_2_mix_aspiration_rate
+            vertical_speed = self._wash_2_vertical_speed
         self._magdeck.disengage()
 
         num_trans, vol_per_trans = uniform_divide(vol, self._wash_max_transfer_vol)
@@ -224,14 +245,16 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
             for n in range(num_trans):
                 if n > 0:
                     tp.dispense(self._wash_air_gap, locationFrom="wash_locs", well_modifier="top(0)")
+
                 tp.aspirate(vol_per_trans, locationFrom="wash_locs")
-                tp.air_gap(self._wash_air_gap)
-                tp.dispense(vol_per_trans+self._wash_air_gap, locationFrom="target", well_modifier="top(0)")
+                tp.move_to(locationFrom="wash_locs", well_modifier="top(0)", speed=vertical_speed)
+                tp.dispense(vol_per_trans, locationFrom="target", well_modifier="top(0)")
                 if n < num_trans-1:
                     tp.air_gap(self._wash_air_gap)
 
             # Mix
             tp.mix(mix_reps, self._wash_mix_vol, locationFrom="mix_locs")
+            tp.move_to(locationFrom="target", well_modifier="top(0)", speed=vertical_speed)
             tp.air_gap(self._wash_air_gap)
             tp.drop_tip()
 
