@@ -119,6 +119,7 @@ class Station(metaclass=StationMeta):
         self._drop_threshold = drop_threshold
         self._dummy_lights = dummy_lights
         self.jupyter = jupyter
+        self.dashboard_input_request = False
         self._language = language
         self._log_filepath = log_filepath.format(time.strftime("%Y_%m_%d__%H_%M_%S"))
         self._log_file_format = log_file_format
@@ -153,15 +154,20 @@ class Station(metaclass=StationMeta):
         self._msg = ""
         self.external = False
         self._run_stage = self._start_at is None
-        self._sound_manager = SoundManager(alarm=os.path.join(os.path.dirname(module_path), 'sounds', 'alarm.mp3'),
-                                           beep=os.path.join(os.path.dirname(module_path), 'sounds', 'beep2.mp3'),
-                                           finish=os.path.join(os.path.dirname(module_path), 'sounds', 'finish.mp3'))
+        self._sound_manager = SoundManager() if self._debug_mode else SoundManager(
+                                    alarm=os.path.join(os.path.dirname(module_path), 'sounds', 'alarm.mp3'),
+                                    beep=os.path.join(os.path.dirname(module_path), 'sounds', 'beep2.mp3'),
+                                    finish=os.path.join(os.path.dirname(module_path), 'sounds', 'finish.mp3'))
 
     def set_external(self, value: bool = True) -> bool:
         self.external = value
         return self.external
     
     set_internal = partialmethod(set_external, value=False)
+
+    def set_dashboard_input(self, value: bool = True) -> bool:
+        self.dashboard_input_request = value
+        return self.dashboard_input_request
     
     def get_msg(self, value: str) -> str:
         return str(type(self).get_message(value, self._language))
@@ -186,17 +192,26 @@ class Station(metaclass=StationMeta):
         if self._start_at == self.stage:
             self._run_stage = True
         self.logger.info("[{}] Stage: {}".format("x" if self._run_stage else " ", self.stage))
-
-        if self._watchdog_enable and not self._ctx.is_simulating():
-            self._watchdog.reset(self._watchdog_timeout)
-
+        self.watchdog_reset(self._watchdog_timeout)
         return self._run_stage
 
     def assert_run_stage_has_been_executed(self):
         ''' If a _start_at_ value was requested, check it at least a corresponding stage was found in protocol.'''
         if not self._run_stage:
             raise Exception("Stage '{}' not found.".format(self._start_at))
-    
+
+    def watchdog_reset(self, timeout_seconds):
+        if self._watchdog_enable and not self._ctx.is_simulating():
+            self._watchdog.reset(timeout_seconds)
+
+    def watchdog_stop(self):
+        if self._watchdog_enable and not self._ctx.is_simulating():
+            self._watchdog.stop()
+
+    def watchdog_start(self, timeout_seconds):
+        if self._watchdog_enable and not self._ctx.is_simulating():
+            self._watchdog.start(timeout_seconds)
+
     @property
     def logger(self) -> logging.getLoggerClass():
         if ((not hasattr(self, "_logger")) or self._logger is None) and self._ctx is not None:
@@ -394,13 +409,10 @@ class Station(metaclass=StationMeta):
     ):
         # Note: changed default homing method to 'move_to_home' Agos, 2021-09-13
 
-        self.status = "pause"
         old_color = self._button.color
         self._button.color = color
 
-        if self._watchdog_enable and not self._ctx.is_simulating():
-            self._ctx.comment("Disabling watchdog")
-            self._watchdog.stop()
+        self.watchdog_stop()
 
         if msg:
             self.msg = msg
@@ -413,11 +425,11 @@ class Station(metaclass=StationMeta):
             lt.start()
         if delay_time > 0:
             self.status = "delay"
-            if self._watchdog_enable and not self._ctx.is_simulating():
-                self._watchdog.start(delay_time * 1.2)
+            self.watchdog_start(delay_time * 1.2)
             self._ctx.delay(delay_time)
-            self._watchdog.stop()
+            self.watchdog_stop()
         if pause:
+            self.status = "pause"
             self._ctx.pause()
             self._ctx.delay(0.1)  # pad to avoid pause leaking
         if blink and not self._ctx.is_simulating():
@@ -436,7 +448,12 @@ class Station(metaclass=StationMeta):
             between()
         self._msg = "{}.\n{}".format(msg, self.get_msg("continue"))
         self.pause(self.msg, blink=False, color=cols[1], home=home[1])
-    
+
+    def request_dashboard_input(self, msg: str, home: Tuple[bool, bool] = (True, False)):
+        self.logger.info("Requesting external barcode")
+        self.dual_pause(msg=msg, home=home, between=self.set_dashboard_input)
+        self.set_dashboard_input(False)
+
     def delay(self,
         mins: float,
         msg: str = "",
